package net.nitus.missatges;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
 * <p>
 * Static methods to generate and show messages using <code>JOptionPane</code>.
 * </p>
 * <p>
 * There are methods to generate dialogs for error, warning, and question
 * messages.
 * </p>
 * <p>
 * Features:
 * </p>
 * <ul>
 * <li>You can generate message boxes passing only the parent component and the
 * message object (i.e. a <code>String</code> or a <code>JLabel</code>.
 * </li>
 * <li>Predefined message box titles for error, warning and question in
 * Catalan. You can change the titles values easily using the apropriate set
 * methods.</li>
 * <li>You can indicate the message must be formatted splitting it into lines
 * of n characters (70 by default) or you can set the line length changing the
 * <code>DEFAULT_LINE_LENGTH</code> value. The message is splitted using word
 * boundaries.
 * </ul>
 */
public class MsgUtils {
	public static String	warningDefaultTitle		= "Av�s:";
	public static String	errorDefaultTitle			= "Error:";
	public static String	questionDefaultTitle	= "Pregunta:";
	public static int			DEFAULT_LINE_LENGTH		= 70;
	

	/**
	 * Splits a string into lines of <code>DEFAULT_LINE_LENGTH</code> length.
	 * <code>DEFAULT_LINE_LENGTH</code>.
	 * 
	 * @param s
	 *          String to be formatted
	 * @return The formatted string
	 * @see MsgUtils#formatMsg(Object, int)
	 */

	public static Object formatMsg(Object o) {
		if (o instanceof String) {
			return formatMsg((String) o, DEFAULT_LINE_LENGTH);
		}
		return o;
	}

	/**
	 * Splits a string into substring adjusting the substring length to a maximum
	 * of <code>len</code> characters. The split point is ever between two
	 * words. The parameter <code>len</code> must be a positive whole number.
	 * <p>
	 * Ignores the existing line feeds into <code>s</code>.
	 * </p>
	 * 
	 * @return The formatted string
	 * @param s
	 *          String to be formatted
	 * @param len
	 *          Splitted line maximum length
	 * @exception java.lang.IllegalArgumentException
	 *              If <code>len</code> is'nt a positive whole number.
	 */
	public static Object formatMsg(Object o, int len) {
		if (!(o instanceof String)) {
			return o;
		}
		String s = (String) o;
		if (s == null)
			return s;
		if (s.length() <= len)
			return s;
		if (len <= 0)
			throw new IllegalArgumentException("len (" + len
					+ ") must be a positive whole number.");
		StringTokenizer st = new StringTokenizer(s);
		StringBuffer sb = new StringBuffer();
		int lineLen = 0;
		while (st.hasMoreElements()) {
			String w = st.nextToken();
			lineLen = lineLen + 1 + w.length();
			if (lineLen > len) {
				sb.append('\n');
				sb.append(w);
				lineLen = w.length();
				continue;
			}
			if (sb.length() != 0)
				sb.append(' ');
			sb.append(w);
		} // while
		return sb.toString();
	}

	/**
	 * @return Returns the warningDefaultTitle.
	 */
	public static String getWarningDefaultTitle() {
		return warningDefaultTitle;
	}

	/**
	 * @return Returns the errorDefaultTitle.
	 */
	public static String getErrorDefaultTitle() {
		return errorDefaultTitle;
	}

	/**
	 * Sets the error messages dialogs title.
	 * 
	 * @param errorDefaultTitle
	 *          The errorDefaultTitle to set.
	 */
	public static void setErrorDefaultTitle(String errorDefaultTitle) {
		MsgUtils.errorDefaultTitle = errorDefaultTitle;
	}

	/**
	 * Sets the warning messages dialogs title.
	 * 
	 * @param newValue
	 *          The questionDefaultTitle to set.
	 */
	public static void setWarningDefaultTitle(String newValue) {
		MsgUtils.warningDefaultTitle = newValue;
	}

	/**
	 * @return Returns the questionDefaultTitle.
	 */
	public static String getQuestionDefaultTitle() {
		return questionDefaultTitle;
	}

	/**
	 * Sets the question messages dialogs title.
	 * 
	 * @param questionDefaultTitle
	 *          The questionDefaultTitle to set.
	 */
	public static void setQuestionDefaultTitle(String questionDefaultTitle) {
		MsgUtils.questionDefaultTitle = questionDefaultTitle;
	}

	/**
	 * Displays an error dialog using the
	 * <code>JOptionPane.showMessageDialog()</code> method.
	 * <p>
	 * The dialog title is the <code>errorDefaultTitle</code>.
	 * </p>
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param msg
	 *          java.lang.Object
	 */
	public static void showError(java.awt.Component comp, Object msg) {
		int msgType = JOptionPane.ERROR_MESSAGE;
		JOptionPane.showMessageDialog(comp, msg, errorDefaultTitle, msgType);
		return;
	}

	/**
	 * Displays an error dialog using the
	 * <code>JOptionPane.showMessageDialog()</code> method.
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param title
	 *          java.lang.String
	 * @param msg
	 *          java.lang.Object
	 */
	public static void showError(java.awt.Component comp, String title, Object msg) {
		int msgType = JOptionPane.ERROR_MESSAGE;
		JOptionPane.showMessageDialog(comp, msg, title, msgType);
		return;
	}

	/**
	 * Displays an error dialog using the
	 * <code>JOptionPane.showMessageDialog()</code> method.
	 * <p>
	 * You can specify if the <code>msg</code> must be formatted or not.
	 * </p>
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param title
	 *          java.lang.String
	 * @param msg
	 *          java.lang.Object
	 * @param formated
	 *          boolean
	 */
	public static void showError(java.awt.Component comp, String title,
			Object msg, boolean formated) {
		if (formated)
			msg = formatMsg(msg);
		showError(comp, title, msg);
	}

	/**
	 * Displays an error dialog using the
	 * <code>JOptionPane.showMessageDialog()</code> method.
	 * <p>
	 * The dialog title is the <code>errorDefaultTitle</code>.
	 * </p>
	 * <p>
	 * You can specify if the <code>msg</code> must be formatted or not.
	 * </p>
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param msg
	 *          java.lang.Object
	 * @param formated
	 *          boolean
	 */
	public static void showError(java.awt.Component comp, Object msg,
			boolean formated) {
		if (formated)
			msg = MsgUtils.formatMsg(msg);
		showError(comp, msg);
	}

	/**
	 * Mostra una caixa de di�leg d'av�s (Usa una <code>JOptionPane</code>).
	 * <p>
	 * El t�tol de la caixa de di�leg per defecte �s <code>Aviso:</code>.
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param msg
	 *          java.lang.Object
	 */
	public static void showWarning(java.awt.Component comp, Object msg) {
		int msgType = JOptionPane.WARNING_MESSAGE;
		JOptionPane.showMessageDialog(comp, msg, warningDefaultTitle, msgType);
	}

	/**
	 * Mostra una caixa de di�leg d'av�s (Usa una <code>JOptionPane</code>).
	 * 
	 * @param comp
	 *          java.awt.Component
	 * @param title
	 *          java.lang.String
	 * @param msg
	 *          java.lang.Object
	 */
	public static void showWarning(java.awt.Component comp, String title, 
			Object msg) {
		int msgType = JOptionPane.WARNING_MESSAGE;
		JOptionPane.showMessageDialog(comp, msg, title, msgType);
		return;
	}

	/**
	 * Mostra una caixa de di�leg d'av�s (Usa una <code>JOptionPane</code>).
	 * <p>
	 * El t�tol de la caixa de di�leg per defecte �s <code>Aviso:</code>.
	 * <p>
	 * Permet especificar si cal formatar el missatge o no. Creation date:
	 * (28/7/01 18:30:40)
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @param formatted
	 *          boolean
	 */
	public static void showWarning(java.awt.Component comp, Object msg,
			boolean formatted) {
		if (formatted)
			msg = formatMsg(msg);
		showWarning(comp, msg);
	}

	/**
	 * Mostra una caixa de di�leg d'av�s (Usa una <code>JOptionPane</code>).
	 * <p>
	 * El t�tol de la caixa de di�leg per defecte �s <code>Aviso:</code>.
	 * <p>
	 * Permet especificar si cal formatar el missatge o no. Creation date:
	 * (28/7/01 18:30:40)
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @param formatted
	 *          boolean
	 */
	public static void showWarning(java.awt.Component comp, String title, Object msg,
			boolean formatted) {
		if (formatted)
			msg = formatMsg(msg);
		showWarning(comp, title, msg);
	}

	/**
	 * Shows a confirm dialog using <code>JOptionPane.showConfirmDialog()</code>
	 * method.
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @param title
	 *          The dialog title
	 * @return An <code>int</code> indicating the option selected by the user
	 */
	public static int showQuestion(java.awt.Component comp, String title, Object msg) {
		return showQuestion(comp, title, msg, false);
	}

	/**
	 * Shows a confirm dialog using <code>JOptionPane.showConfirmDialog()</code>
	 * method and the default question title.
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @return An <code>int</code> indicating the option selected by the user
	 */
	
	public static int showQuestion(java.awt.Component comp, Object msg) {
		return showQuestion(comp, questionDefaultTitle, msg, false);
	}

	/**
	 * Shows a confirm dialog using <code>JOptionPane.showConfirmDialog()</code>
	 * method and the default question title.
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @param formatted
	 *          Indicates if the msg will be formatted with
	 *          <code>DEFAULT_LINE_LENGTH</code> (true) or not (false)
	 * @return An <code>int</code> indicating the option selected by the user
	 */
	
	public static int showQuestion(java.awt.Component comp, Object msg,
			boolean formatted) {
		return showQuestion(comp, questionDefaultTitle,  msg, formatted);
	}

	/**
	 * Shows a confirm dialog using <code>JOptionPane.showConfirmDialog()</code>
	 * method and the default question title.
	 * 
	 * @param comp
	 *          Determines the Frame in which the dialog is displayed; if null, or
	 *          if the parentComponent has no Frame, a default Frame is used
	 * @param msg
	 *          Object containing the message to display
	 * @param title
	 *          The dialog title
	 * @param formatted
	 *          Indicates if the msg will be formatted with
	 *          <code>DEFAULT_LINE_LENGTH</code> (true) or not (false)
	 * @return An <code>int</code> indicating the option selected by the user
	 */
	public static int showQuestion(Component comp, String title, Object msg,
			 boolean formatted) {
		/*
	    if (!formatted) {
			return JOptionPane.showConfirmDialog(comp, msg, title,
					JOptionPane.YES_NO_OPTION);
		} else {
			return JOptionPane.showConfirmDialog(comp, formatMsg(msg), title,
					JOptionPane.YES_NO_OPTION);
	     */
	    final JOptionPane question;
	    if (!formatted) {
	        question = new JOptionPane(msg,JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	    }
	    else {
	        question = new JOptionPane(formatMsg(msg),JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	    }
	    
	    final JDialog d = question.createDialog(comp,title);
	    // We are looking the buttons and we want to add a key listener to each them.
	    JPanel p =(JPanel) question.getComponents()[1];

	    JButton si = (JButton) p.getComponents()[0];
	    final int siCode = si.getMnemonic();
	    
	    JButton no = (JButton) p.getComponents()[1];
	    final int noCode = no.getMnemonic();

		KeyAdapter tecles = new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
			    if (e.getKeyCode()==siCode) {
					question.setValue(new Integer(JOptionPane.YES_OPTION));
					e.consume();
					d.dispose();
				}
			    if (e.getKeyCode() == noCode) {
					question.setValue(new Integer(JOptionPane.NO_OPTION));
					e.consume();
					d.dispose();
				}
			};
		};
		si.addKeyListener(tecles);
		no.addKeyListener(tecles);

		d.setVisible(true);

	    Object val = question.getValue();
	    if (val == null) return JOptionPane.CLOSED_OPTION;
	    int answer = ((Integer) question.getValue()).intValue();
	    
	    return answer;
	}
	
	/**
	 * Pita i mostra un missatge amb el t�tol "No implementat:" i el
	 * missatge que ens passen com a par�metre.
	 * @param missatge
	 */
	public static void noFet(String missatge){
	    Toolkit.getDefaultToolkit().beep();
	    MsgUtils.showError(null,"No implementat:",missatge,true);
	    return;

	}

	/**
	 * Pita i mostra un missatge amb el t�tol "No implementat:" i el
	 * missatge "Funci� no implementada. Contacteu amb l'administrador."
	 */
	public static void noFet(){
	    noFet("Funci� no implementada. Contacteu amb l'administrador.");
	    return;
	}
}

